
function Check-SyncMethods()
{ 
    $retValue = $false

    $TextPattern=[regex]'\/\/\/ <summary>*([\r\n][ \t]+\/\/\/.*)+[\r\n][ \t]+public static.*\)[\r\n][ \t]+\{[\r\n].*\.GetAwaiter\(\)\.GetResult\(\);[\r\n][ \t]+\}[\r\n][\r\n][ \t]+'

    Get-ChildItem -Recurse -Filter "*.cs" | Where-Object { $_.Attributes -ne "Directory"} | ForEach-Object { 
        $text = (Get-Content $_.FullName) -join "`n"

        if($text -match $TextPattern) {
            Write-Host "[ERROR]: Found Sync Method in file: $($_.FullName)"
            $retValue = $true
        }
    }
    return $retValue;
}    

function Check-LicenseHeader()
{
    $retValue = $false
    $licenseHeader = "// Copyright (c) Microsoft Corporation. All rights reserved."
    $licenseHeaderAutoGen = "// <auto-generated> // Copyright (c) Microsoft Corporation. All rights reserved."

    Get-ChildItem -Recurse -Filter "*.cs" | Where-Object { $_.Attributes -ne "Directory"} | ForEach-Object { 
        $text = (Get-Content $_.FullName -First 2) -join " "

        if((-Not $text.StartsWith($licenseHeader)) -And (-Not $text.StartsWith($licenseHeaderAutoGen))) {
            Write-Host "[ERROR]: File does not contain valid License Header: $($_.FullName)"
            $retValue = $true
        }
    }
    return $retValue;
}

$syncMethods = Check-SyncMethods
$licenseHeaders = Check-LicenseHeader

if($syncMethods -Or $licenseHeaders) {
    Write-Host "[FAILED]: Style Check failed..."
    exit(-1)
}

Write-Host "[SUCCESS]: Style Check passed..."
exit(0)
