// Copyright (c) Microsoft Corporation. All rights reserved.
// Licensed under the MIT License. See License.txt in the project root for license information.

namespace Microsoft.Azure.Management.Batch.Fluent
{
    using Management.Batch;
    using Microsoft.Rest.Azure;
    using ResourceManager.Fluent.Core;
    using System.Collections.Generic;
    using System.Collections.ObjectModel;
    using System.Linq;

    /// <summary>
    /// Represents a applicationPackage collection associated with a application.
    /// </summary>
    ///GENTHASH:Y29tLm1pY3Jvc29mdC5henVyZS5tYW5hZ2VtZW50LmJhdGNoLmltcGxlbWVudGF0aW9uLkFwcGxpY2F0aW9uUGFja2FnZXNJbXBs
    internal partial class ApplicationPackagesImpl :
        ExternalChildResourcesCached<ApplicationPackageImpl,
            IApplicationPackage,
            Management.Batch.Fluent.Models.ApplicationPackageInner,
            IApplication,
            ApplicationImpl>
    {
        private IApplicationPackageOperations client;
        private ApplicationImpl parent;

        ///GENMHASH:1E4CA209B0AD4710E5C7F91ECC84511C:5B3AB401341688B60693658520A75514
        internal ApplicationPackagesImpl(IApplicationPackageOperations client, ApplicationImpl parent)
            : base(parent, "ApplicationPackage")
        {
            this.client = client;
            this.parent = parent;
            CacheCollection();
        }

        ///GENMHASH:8ACFB0E23F5F24AD384313679B65F404:E3D9E623E212362C450DC90E92DF3FCC
        public ApplicationPackageImpl Define(string name)
        {
            return PrepareDefine(name);
        }

        ///GENMHASH:FC8ECF797E9AF86E82C3899A3D5C00BB:F0D1229265F1637BBE32E524D479B677
        internal void Remove(string applicationPackageName)
        {
            PrepareRemove(applicationPackageName);
        }

        ///GENMHASH:6A122C62EB559D6E6E53725061B422FB:31266E0F5C59A745DCBC67FE70DED1E8
        protected override IList<ApplicationPackageImpl> ListChildResources()
        {
            var childResources = new List<ApplicationPackageImpl>();

            try
            {
                var applicationPackageList = Extensions.Synchronize(() => parent.Parent.Manager.Inner.ApplicationPackage.ListAsync(parent.Parent.ResourceGroupName, parent.Parent.Name, parent.Name()));

                foreach (var applicationPackage in applicationPackageList)
                {
                    childResources.Add(new ApplicationPackageImpl(applicationPackage.Name, parent, applicationPackage, client));
                }
            } catch (CloudException) {
                //ignore the exception when creating application package during application definition stage
            }

            return childResources;
        }

        ///GENMHASH:8E8DA5B84731A2D412247D25A544C502:2574C4932A56E8A550A63AD4C374CD50
        protected override ApplicationPackageImpl NewChildResource(string name)
        {
            return ApplicationPackageImpl.NewApplicationPackage(name, parent, client);
        }

        ///GENMHASH:FDBA2803E9882B732C94FBD64DC9D13B:3CEA2921F291B19D8593A8BD0C8FFC02
        internal void AddApplicationPackage(ApplicationPackageImpl applicationPackage)
        {
            AddChildResource(applicationPackage);
        }

        ///GENMHASH:310B2185D2F2431DF2BBDBC06E585C74:F487B2409E11AB3B3255E980C7B88B89
        internal IReadOnlyDictionary<string, Microsoft.Azure.Management.Batch.Fluent.IApplicationPackage> AsMap()
        {
            var result = new Dictionary<string, IApplicationPackage>();

            foreach (var entry in Collection)
            {
                ApplicationPackageImpl applicationPackage = entry.Value;
                result.Add(entry.Key, applicationPackage);
            }

            return new ReadOnlyDictionary<string, IApplicationPackage>(result);
        }
    }
}
