// <auto-generated>
// Copyright (c) Microsoft Corporation. All rights reserved.
// Licensed under the MIT License. See License.txt in the project root for
// license information.
//
// Code generated by Microsoft (R) AutoRest Code Generator.
// </auto-generated>

namespace Microsoft.Azure.Management.BatchAI.Fluent
{
    using Microsoft.Rest;
    using Microsoft.Rest.Azure;
    using Models;
    using System.Threading;
    using System.Threading.Tasks;

    /// <summary>
    /// Extension methods for JobsOperations.
    /// </summary>
    public static partial class JobsOperationsExtensions
    {
            /// <summary>
            /// Gets a list of Jobs within the specified Experiment.
            /// </summary>
            /// <param name='operations'>
            /// The operations group for this extension method.
            /// </param>
            /// <param name='resourceGroupName'>
            /// Name of the resource group to which the resource belongs.
            /// </param>
            /// <param name='workspaceName'>
            /// The name of the workspace. Workspace names can only contain a combination
            /// of alphanumeric characters along with dash (-) and underscore (_). The name
            /// must be from 1 through 64 characters long.
            /// </param>
            /// <param name='experimentName'>
            /// The name of the experiment. Experiment names can only contain a combination
            /// of alphanumeric characters along with dash (-) and underscore (_). The name
            /// must be from 1 through 64 characters long.
            /// </param>
            /// <param name='jobsListByExperimentOptions'>
            /// Additional parameters for the operation
            /// </param>
            /// <param name='cancellationToken'>
            /// The cancellation token.
            /// </param>
            public static async Task<IPage<JobInner>> ListByExperimentAsync(this IJobsOperations operations, string resourceGroupName, string workspaceName, string experimentName, JobsListByExperimentOptions jobsListByExperimentOptions = default(JobsListByExperimentOptions), CancellationToken cancellationToken = default(CancellationToken))
            {
                using (var _result = await operations.ListByExperimentWithHttpMessagesAsync(resourceGroupName, workspaceName, experimentName, jobsListByExperimentOptions, null, cancellationToken).ConfigureAwait(false))
                {
                    return _result.Body;
                }
            }

            /// <summary>
            /// Creates a Job in the given Experiment.
            /// </summary>
            /// <param name='operations'>
            /// The operations group for this extension method.
            /// </param>
            /// <param name='resourceGroupName'>
            /// Name of the resource group to which the resource belongs.
            /// </param>
            /// <param name='workspaceName'>
            /// The name of the workspace. Workspace names can only contain a combination
            /// of alphanumeric characters along with dash (-) and underscore (_). The name
            /// must be from 1 through 64 characters long.
            /// </param>
            /// <param name='experimentName'>
            /// The name of the experiment. Experiment names can only contain a combination
            /// of alphanumeric characters along with dash (-) and underscore (_). The name
            /// must be from 1 through 64 characters long.
            /// </param>
            /// <param name='jobName'>
            /// The name of the job within the specified resource group. Job names can only
            /// contain a combination of alphanumeric characters along with dash (-) and
            /// underscore (_). The name must be from 1 through 64 characters long.
            /// </param>
            /// <param name='parameters'>
            /// The parameters to provide for job creation.
            /// </param>
            /// <param name='cancellationToken'>
            /// The cancellation token.
            /// </param>
            public static async Task<JobInner> CreateAsync(this IJobsOperations operations, string resourceGroupName, string workspaceName, string experimentName, string jobName, JobCreateParameters parameters, CancellationToken cancellationToken = default(CancellationToken))
            {
                using (var _result = await operations.CreateWithHttpMessagesAsync(resourceGroupName, workspaceName, experimentName, jobName, parameters, null, cancellationToken).ConfigureAwait(false))
                {
                    return _result.Body;
                }
            }

            /// <summary>
            /// Deletes a Job.
            /// </summary>
            /// <param name='operations'>
            /// The operations group for this extension method.
            /// </param>
            /// <param name='resourceGroupName'>
            /// Name of the resource group to which the resource belongs.
            /// </param>
            /// <param name='workspaceName'>
            /// The name of the workspace. Workspace names can only contain a combination
            /// of alphanumeric characters along with dash (-) and underscore (_). The name
            /// must be from 1 through 64 characters long.
            /// </param>
            /// <param name='experimentName'>
            /// The name of the experiment. Experiment names can only contain a combination
            /// of alphanumeric characters along with dash (-) and underscore (_). The name
            /// must be from 1 through 64 characters long.
            /// </param>
            /// <param name='jobName'>
            /// The name of the job within the specified resource group. Job names can only
            /// contain a combination of alphanumeric characters along with dash (-) and
            /// underscore (_). The name must be from 1 through 64 characters long.
            /// </param>
            /// <param name='cancellationToken'>
            /// The cancellation token.
            /// </param>
            public static async Task DeleteAsync(this IJobsOperations operations, string resourceGroupName, string workspaceName, string experimentName, string jobName, CancellationToken cancellationToken = default(CancellationToken))
            {
                (await operations.DeleteWithHttpMessagesAsync(resourceGroupName, workspaceName, experimentName, jobName, null, cancellationToken).ConfigureAwait(false)).Dispose();
            }

            /// <summary>
            /// Gets information about a Job.
            /// </summary>
            /// <param name='operations'>
            /// The operations group for this extension method.
            /// </param>
            /// <param name='resourceGroupName'>
            /// Name of the resource group to which the resource belongs.
            /// </param>
            /// <param name='workspaceName'>
            /// The name of the workspace. Workspace names can only contain a combination
            /// of alphanumeric characters along with dash (-) and underscore (_). The name
            /// must be from 1 through 64 characters long.
            /// </param>
            /// <param name='experimentName'>
            /// The name of the experiment. Experiment names can only contain a combination
            /// of alphanumeric characters along with dash (-) and underscore (_). The name
            /// must be from 1 through 64 characters long.
            /// </param>
            /// <param name='jobName'>
            /// The name of the job within the specified resource group. Job names can only
            /// contain a combination of alphanumeric characters along with dash (-) and
            /// underscore (_). The name must be from 1 through 64 characters long.
            /// </param>
            /// <param name='cancellationToken'>
            /// The cancellation token.
            /// </param>
            public static async Task<JobInner> GetAsync(this IJobsOperations operations, string resourceGroupName, string workspaceName, string experimentName, string jobName, CancellationToken cancellationToken = default(CancellationToken))
            {
                using (var _result = await operations.GetWithHttpMessagesAsync(resourceGroupName, workspaceName, experimentName, jobName, null, cancellationToken).ConfigureAwait(false))
                {
                    return _result.Body;
                }
            }

            /// <summary>
            /// List all directories and files inside the given directory of the Job's
            /// output directory (if the output directory is on Azure File Share or Azure
            /// Storage Container).
            /// </summary>
            /// <param name='operations'>
            /// The operations group for this extension method.
            /// </param>
            /// <param name='resourceGroupName'>
            /// Name of the resource group to which the resource belongs.
            /// </param>
            /// <param name='workspaceName'>
            /// The name of the workspace. Workspace names can only contain a combination
            /// of alphanumeric characters along with dash (-) and underscore (_). The name
            /// must be from 1 through 64 characters long.
            /// </param>
            /// <param name='experimentName'>
            /// The name of the experiment. Experiment names can only contain a combination
            /// of alphanumeric characters along with dash (-) and underscore (_). The name
            /// must be from 1 through 64 characters long.
            /// </param>
            /// <param name='jobName'>
            /// The name of the job within the specified resource group. Job names can only
            /// contain a combination of alphanumeric characters along with dash (-) and
            /// underscore (_). The name must be from 1 through 64 characters long.
            /// </param>
            /// <param name='jobsListOutputFilesOptions'>
            /// Additional parameters for the operation
            /// </param>
            /// <param name='cancellationToken'>
            /// The cancellation token.
            /// </param>
            public static async Task<IPage<File>> ListOutputFilesAsync(this IJobsOperations operations, string resourceGroupName, string workspaceName, string experimentName, string jobName, JobsListOutputFilesOptions jobsListOutputFilesOptions, CancellationToken cancellationToken = default(CancellationToken))
            {
                using (var _result = await operations.ListOutputFilesWithHttpMessagesAsync(resourceGroupName, workspaceName, experimentName, jobName, jobsListOutputFilesOptions, null, cancellationToken).ConfigureAwait(false))
                {
                    return _result.Body;
                }
            }

            /// <summary>
            /// Gets a list of currently existing nodes which were used for the Job
            /// execution. The returned information contains the node ID, its public IP and
            /// SSH port.
            /// </summary>
            /// <param name='operations'>
            /// The operations group for this extension method.
            /// </param>
            /// <param name='resourceGroupName'>
            /// Name of the resource group to which the resource belongs.
            /// </param>
            /// <param name='workspaceName'>
            /// The name of the workspace. Workspace names can only contain a combination
            /// of alphanumeric characters along with dash (-) and underscore (_). The name
            /// must be from 1 through 64 characters long.
            /// </param>
            /// <param name='experimentName'>
            /// The name of the experiment. Experiment names can only contain a combination
            /// of alphanumeric characters along with dash (-) and underscore (_). The name
            /// must be from 1 through 64 characters long.
            /// </param>
            /// <param name='jobName'>
            /// The name of the job within the specified resource group. Job names can only
            /// contain a combination of alphanumeric characters along with dash (-) and
            /// underscore (_). The name must be from 1 through 64 characters long.
            /// </param>
            /// <param name='cancellationToken'>
            /// The cancellation token.
            /// </param>
            public static async Task<IPage<RemoteLoginInformation>> ListRemoteLoginInformationAsync(this IJobsOperations operations, string resourceGroupName, string workspaceName, string experimentName, string jobName, CancellationToken cancellationToken = default(CancellationToken))
            {
                using (var _result = await operations.ListRemoteLoginInformationWithHttpMessagesAsync(resourceGroupName, workspaceName, experimentName, jobName, null, cancellationToken).ConfigureAwait(false))
                {
                    return _result.Body;
                }
            }

            /// <summary>
            /// Terminates a job.
            /// </summary>
            /// <param name='operations'>
            /// The operations group for this extension method.
            /// </param>
            /// <param name='resourceGroupName'>
            /// Name of the resource group to which the resource belongs.
            /// </param>
            /// <param name='workspaceName'>
            /// The name of the workspace. Workspace names can only contain a combination
            /// of alphanumeric characters along with dash (-) and underscore (_). The name
            /// must be from 1 through 64 characters long.
            /// </param>
            /// <param name='experimentName'>
            /// The name of the experiment. Experiment names can only contain a combination
            /// of alphanumeric characters along with dash (-) and underscore (_). The name
            /// must be from 1 through 64 characters long.
            /// </param>
            /// <param name='jobName'>
            /// The name of the job within the specified resource group. Job names can only
            /// contain a combination of alphanumeric characters along with dash (-) and
            /// underscore (_). The name must be from 1 through 64 characters long.
            /// </param>
            /// <param name='cancellationToken'>
            /// The cancellation token.
            /// </param>
            public static async Task TerminateAsync(this IJobsOperations operations, string resourceGroupName, string workspaceName, string experimentName, string jobName, CancellationToken cancellationToken = default(CancellationToken))
            {
                (await operations.TerminateWithHttpMessagesAsync(resourceGroupName, workspaceName, experimentName, jobName, null, cancellationToken).ConfigureAwait(false)).Dispose();
            }

            /// <summary>
            /// Creates a Job in the given Experiment.
            /// </summary>
            /// <param name='operations'>
            /// The operations group for this extension method.
            /// </param>
            /// <param name='resourceGroupName'>
            /// Name of the resource group to which the resource belongs.
            /// </param>
            /// <param name='workspaceName'>
            /// The name of the workspace. Workspace names can only contain a combination
            /// of alphanumeric characters along with dash (-) and underscore (_). The name
            /// must be from 1 through 64 characters long.
            /// </param>
            /// <param name='experimentName'>
            /// The name of the experiment. Experiment names can only contain a combination
            /// of alphanumeric characters along with dash (-) and underscore (_). The name
            /// must be from 1 through 64 characters long.
            /// </param>
            /// <param name='jobName'>
            /// The name of the job within the specified resource group. Job names can only
            /// contain a combination of alphanumeric characters along with dash (-) and
            /// underscore (_). The name must be from 1 through 64 characters long.
            /// </param>
            /// <param name='parameters'>
            /// The parameters to provide for job creation.
            /// </param>
            /// <param name='cancellationToken'>
            /// The cancellation token.
            /// </param>
            public static async Task<JobInner> BeginCreateAsync(this IJobsOperations operations, string resourceGroupName, string workspaceName, string experimentName, string jobName, JobCreateParameters parameters, CancellationToken cancellationToken = default(CancellationToken))
            {
                using (var _result = await operations.BeginCreateWithHttpMessagesAsync(resourceGroupName, workspaceName, experimentName, jobName, parameters, null, cancellationToken).ConfigureAwait(false))
                {
                    return _result.Body;
                }
            }

            /// <summary>
            /// Deletes a Job.
            /// </summary>
            /// <param name='operations'>
            /// The operations group for this extension method.
            /// </param>
            /// <param name='resourceGroupName'>
            /// Name of the resource group to which the resource belongs.
            /// </param>
            /// <param name='workspaceName'>
            /// The name of the workspace. Workspace names can only contain a combination
            /// of alphanumeric characters along with dash (-) and underscore (_). The name
            /// must be from 1 through 64 characters long.
            /// </param>
            /// <param name='experimentName'>
            /// The name of the experiment. Experiment names can only contain a combination
            /// of alphanumeric characters along with dash (-) and underscore (_). The name
            /// must be from 1 through 64 characters long.
            /// </param>
            /// <param name='jobName'>
            /// The name of the job within the specified resource group. Job names can only
            /// contain a combination of alphanumeric characters along with dash (-) and
            /// underscore (_). The name must be from 1 through 64 characters long.
            /// </param>
            /// <param name='cancellationToken'>
            /// The cancellation token.
            /// </param>
            public static async Task BeginDeleteAsync(this IJobsOperations operations, string resourceGroupName, string workspaceName, string experimentName, string jobName, CancellationToken cancellationToken = default(CancellationToken))
            {
                (await operations.BeginDeleteWithHttpMessagesAsync(resourceGroupName, workspaceName, experimentName, jobName, null, cancellationToken).ConfigureAwait(false)).Dispose();
            }

            /// <summary>
            /// Terminates a job.
            /// </summary>
            /// <param name='operations'>
            /// The operations group for this extension method.
            /// </param>
            /// <param name='resourceGroupName'>
            /// Name of the resource group to which the resource belongs.
            /// </param>
            /// <param name='workspaceName'>
            /// The name of the workspace. Workspace names can only contain a combination
            /// of alphanumeric characters along with dash (-) and underscore (_). The name
            /// must be from 1 through 64 characters long.
            /// </param>
            /// <param name='experimentName'>
            /// The name of the experiment. Experiment names can only contain a combination
            /// of alphanumeric characters along with dash (-) and underscore (_). The name
            /// must be from 1 through 64 characters long.
            /// </param>
            /// <param name='jobName'>
            /// The name of the job within the specified resource group. Job names can only
            /// contain a combination of alphanumeric characters along with dash (-) and
            /// underscore (_). The name must be from 1 through 64 characters long.
            /// </param>
            /// <param name='cancellationToken'>
            /// The cancellation token.
            /// </param>
            public static async Task BeginTerminateAsync(this IJobsOperations operations, string resourceGroupName, string workspaceName, string experimentName, string jobName, CancellationToken cancellationToken = default(CancellationToken))
            {
                (await operations.BeginTerminateWithHttpMessagesAsync(resourceGroupName, workspaceName, experimentName, jobName, null, cancellationToken).ConfigureAwait(false)).Dispose();
            }

            /// <summary>
            /// Gets a list of Jobs within the specified Experiment.
            /// </summary>
            /// <param name='operations'>
            /// The operations group for this extension method.
            /// </param>
            /// <param name='nextPageLink'>
            /// The NextLink from the previous successful call to List operation.
            /// </param>
            /// <param name='cancellationToken'>
            /// The cancellation token.
            /// </param>
            public static async Task<IPage<JobInner>> ListByExperimentNextAsync(this IJobsOperations operations, string nextPageLink, CancellationToken cancellationToken = default(CancellationToken))
            {
                using (var _result = await operations.ListByExperimentNextWithHttpMessagesAsync(nextPageLink, null, cancellationToken).ConfigureAwait(false))
                {
                    return _result.Body;
                }
            }

            /// <summary>
            /// List all directories and files inside the given directory of the Job's
            /// output directory (if the output directory is on Azure File Share or Azure
            /// Storage Container).
            /// </summary>
            /// <param name='operations'>
            /// The operations group for this extension method.
            /// </param>
            /// <param name='nextPageLink'>
            /// The NextLink from the previous successful call to List operation.
            /// </param>
            /// <param name='cancellationToken'>
            /// The cancellation token.
            /// </param>
            public static async Task<IPage<File>> ListOutputFilesNextAsync(this IJobsOperations operations, string nextPageLink, CancellationToken cancellationToken = default(CancellationToken))
            {
                using (var _result = await operations.ListOutputFilesNextWithHttpMessagesAsync(nextPageLink, null, cancellationToken).ConfigureAwait(false))
                {
                    return _result.Body;
                }
            }

            /// <summary>
            /// Gets a list of currently existing nodes which were used for the Job
            /// execution. The returned information contains the node ID, its public IP and
            /// SSH port.
            /// </summary>
            /// <param name='operations'>
            /// The operations group for this extension method.
            /// </param>
            /// <param name='nextPageLink'>
            /// The NextLink from the previous successful call to List operation.
            /// </param>
            /// <param name='cancellationToken'>
            /// The cancellation token.
            /// </param>
            public static async Task<IPage<RemoteLoginInformation>> ListRemoteLoginInformationNextAsync(this IJobsOperations operations, string nextPageLink, CancellationToken cancellationToken = default(CancellationToken))
            {
                using (var _result = await operations.ListRemoteLoginInformationNextWithHttpMessagesAsync(nextPageLink, null, cancellationToken).ConfigureAwait(false))
                {
                    return _result.Body;
                }
            }

    }
}
