// <auto-generated>
// Copyright (c) Microsoft Corporation. All rights reserved.
// Licensed under the MIT License. See License.txt in the project root for
// license information.
//
// Code generated by Microsoft (R) AutoRest Code Generator.
// </auto-generated>

namespace Microsoft.Azure.Management.BatchAI.Fluent.Models
{
    using Microsoft.Rest;
    using Newtonsoft.Json;
    using System.Linq;

    /// <summary>
    /// Azure File Share mounting configuration.
    /// </summary>
    public partial class AzureFileShareReference
    {
        /// <summary>
        /// Initializes a new instance of the AzureFileShareReference class.
        /// </summary>
        public AzureFileShareReference()
        {
            CustomInit();
        }

        /// <summary>
        /// Initializes a new instance of the AzureFileShareReference class.
        /// </summary>
        /// <param name="accountName">Account name.</param>
        /// <param name="azureFileUrl">Azure File URL.</param>
        /// <param name="credentials">Credentials.</param>
        /// <param name="relativeMountPath">Relative mount path.</param>
        /// <param name="fileMode">File mode.</param>
        /// <param name="directoryMode">Directory mode.</param>
        public AzureFileShareReference(string accountName, string azureFileUrl, AzureStorageCredentialsInfo credentials, string relativeMountPath, string fileMode = default(string), string directoryMode = default(string))
        {
            AccountName = accountName;
            AzureFileUrl = azureFileUrl;
            Credentials = credentials;
            RelativeMountPath = relativeMountPath;
            FileMode = fileMode;
            DirectoryMode = directoryMode;
            CustomInit();
        }

        /// <summary>
        /// An initialization method that performs custom operations like setting defaults
        /// </summary>
        partial void CustomInit();

        /// <summary>
        /// Gets or sets account name.
        /// </summary>
        /// <remarks>
        /// Name of the Azure storage account.
        /// </remarks>
        [JsonProperty(PropertyName = "accountName")]
        public string AccountName { get; set; }

        /// <summary>
        /// Gets or sets azure File URL.
        /// </summary>
        /// <remarks>
        /// URL to access the Azure File.
        /// </remarks>
        [JsonProperty(PropertyName = "azureFileUrl")]
        public string AzureFileUrl { get; set; }

        /// <summary>
        /// Gets or sets credentials.
        /// </summary>
        /// <remarks>
        /// Information about the Azure storage credentials.
        /// </remarks>
        [JsonProperty(PropertyName = "credentials")]
        public AzureStorageCredentialsInfo Credentials { get; set; }

        /// <summary>
        /// Gets or sets relative mount path.
        /// </summary>
        /// <remarks>
        /// The relative path on the compute node where the Azure File share
        /// will be mounted. Note that all cluster level file shares will be
        /// mounted under $AZ_BATCHAI_MOUNT_ROOT location and all job level
        /// file shares will be mounted under $AZ_BATCHAI_JOB_MOUNT_ROOT.
        /// </remarks>
        [JsonProperty(PropertyName = "relativeMountPath")]
        public string RelativeMountPath { get; set; }

        /// <summary>
        /// Gets or sets file mode.
        /// </summary>
        /// <remarks>
        /// File mode for files on the mounted file share. Default value: 0777.
        /// </remarks>
        [JsonProperty(PropertyName = "fileMode")]
        public string FileMode { get; set; }

        /// <summary>
        /// Gets or sets directory mode.
        /// </summary>
        /// <remarks>
        /// File mode for directories on the mounted file share. Default value:
        /// 0777.
        /// </remarks>
        [JsonProperty(PropertyName = "directoryMode")]
        public string DirectoryMode { get; set; }

        /// <summary>
        /// Validate the object.
        /// </summary>
        /// <exception cref="ValidationException">
        /// Thrown if validation fails
        /// </exception>
        public virtual void Validate()
        {
            if (AccountName == null)
            {
                throw new ValidationException(ValidationRules.CannotBeNull, "AccountName");
            }
            if (AzureFileUrl == null)
            {
                throw new ValidationException(ValidationRules.CannotBeNull, "AzureFileUrl");
            }
            if (Credentials == null)
            {
                throw new ValidationException(ValidationRules.CannotBeNull, "Credentials");
            }
            if (RelativeMountPath == null)
            {
                throw new ValidationException(ValidationRules.CannotBeNull, "RelativeMountPath");
            }
            if (Credentials != null)
            {
                Credentials.Validate();
            }
        }
    }
}
