// Copyright (c) Microsoft Corporation. All rights reserved.
// Licensed under the MIT License. See License.txt in the project root for license information.

namespace Microsoft.Azure.Management.Compute.Fluent
{
    using Microsoft.Azure.Management.Compute.Fluent.Models;
    using Microsoft.Azure.Management.Compute.Fluent.VirtualMachineScaleSet.Definition;
    using Microsoft.Azure.Management.Compute.Fluent.VirtualMachineScaleSet.Update;
    using Microsoft.Azure.Management.Compute.Fluent.VirtualMachineScaleSetUnmanagedDataDisk.Definition;
    using Microsoft.Azure.Management.Compute.Fluent.VirtualMachineScaleSetUnmanagedDataDisk.DefinitionWithImage;
    using Microsoft.Azure.Management.Compute.Fluent.VirtualMachineScaleSetUnmanagedDataDisk.DefinitionWithNewVhd;
    using Microsoft.Azure.Management.Compute.Fluent.VirtualMachineScaleSetUnmanagedDataDisk.Update;
    using Microsoft.Azure.Management.Compute.Fluent.VirtualMachineScaleSetUnmanagedDataDisk.UpdateDefinition;
    using Microsoft.Azure.Management.ResourceManager.Fluent.Core;
    using Microsoft.Azure.Management.ResourceManager.Fluent.Core.ChildResource.Definition;
    using Microsoft.Azure.Management.ResourceManager.Fluent.Core.ChildResource.Update;
    using System.Collections.Generic;

    internal partial class VirtualMachineScaleSetUnmanagedDataDiskImpl
    {
        /// <summary>
        /// Gets the name of the resource.
        /// </summary>
        string Microsoft.Azure.Management.ResourceManager.Fluent.Core.IHasName.Name
        {
            get
            {
                return this.Name();
            }
        }

        /// <summary>
        /// Attaches the child definition to the parent resource update.
        /// </summary>
        /// <return>The next stage of the parent definition.</return>
        VirtualMachineScaleSet.Update.IWithApply Microsoft.Azure.Management.ResourceManager.Fluent.Core.ChildResource.Update.IInUpdate<VirtualMachineScaleSet.Update.IWithApply>.Attach()
        {
            return this.Attach();
        }

        /// <summary>
        /// Attaches the child definition to the parent resource definiton.
        /// </summary>
        /// <return>The next stage of the parent definition.</return>
        VirtualMachineScaleSet.Definition.IWithUnmanagedCreate Microsoft.Azure.Management.ResourceManager.Fluent.Core.ChildResource.Definition.IInDefinition<VirtualMachineScaleSet.Definition.IWithUnmanagedCreate>.Attach()
        {
            return this.Attach();
        }

        /// <summary>
        /// Specifies the image LUN identifier of the source disk image.
        /// </summary>
        /// <param name="imageLun">The LUN.</param>
        /// <return>The next stage of unmanaged data disk definition.</return>
        VirtualMachineScaleSetUnmanagedDataDisk.Definition.IWithFromImageDiskSettings<VirtualMachineScaleSet.Definition.IWithUnmanagedCreate> VirtualMachineScaleSetUnmanagedDataDisk.Definition.IWithDiskSource<VirtualMachineScaleSet.Definition.IWithUnmanagedCreate>.FromImage(int imageLun)
        {
            return this.FromImage(imageLun);
        }

        /// <summary>
        /// Specifies the new caching type for the unmanaged data disk.
        /// </summary>
        /// <param name="cachingType">The disk caching type. Possible values include: 'None', 'ReadOnly', 'ReadWrite'.</param>
        /// <return>The next stage of unmanaged data disk update.</return>
        VirtualMachineScaleSetUnmanagedDataDisk.Update.IUpdate VirtualMachineScaleSetUnmanagedDataDisk.Update.IWithDiskCaching.WithCaching(CachingTypes cachingType)
        {
            return this.WithCaching(cachingType);
        }

        /// <summary>
        /// Specifies the caching type for the unmanaged data disk.
        /// </summary>
        /// <param name="cachingType">The disk caching type. Possible values include: 'None', 'ReadOnly', 'ReadWrite'.</param>
        /// <return>The next stage of unmanaged data disk definition.</return>
        VirtualMachineScaleSetUnmanagedDataDisk.Definition.IWithNewVhdDiskSettings<VirtualMachineScaleSet.Definition.IWithUnmanagedCreate> VirtualMachineScaleSetUnmanagedDataDisk.Definition.IWithNewVhdDiskSettings<VirtualMachineScaleSet.Definition.IWithUnmanagedCreate>.WithCaching(CachingTypes cachingType)
        {
            return this.WithCaching(cachingType);
        }

        /// <summary>
        /// Specifies the caching type for the unmanaged data disk.
        /// </summary>
        /// <param name="cachingType">The disk caching type. Possible values include: 'None', 'ReadOnly', 'ReadWrite'.</param>
        /// <return>The next stage of unmanaged data disk definition.</return>
        VirtualMachineScaleSetUnmanagedDataDisk.UpdateDefinition.IWithNewVhdDiskSettings<VirtualMachineScaleSet.Update.IWithApply> VirtualMachineScaleSetUnmanagedDataDisk.UpdateDefinition.IWithNewVhdDiskSettings<VirtualMachineScaleSet.Update.IWithApply>.WithCaching(CachingTypes cachingType)
        {
            return this.WithCaching(cachingType);
        }

        /// <summary>
        /// Specifies the caching type for the unmanaged data disk.
        /// </summary>
        /// <param name="cachingType">The disk caching type. Possible values include: 'None', 'ReadOnly', 'ReadWrite'.</param>
        /// <return>The next stage of unmanaged data disk definition.</return>
        VirtualMachineScaleSetUnmanagedDataDisk.Definition.IWithFromImageDiskSettings<VirtualMachineScaleSet.Definition.IWithUnmanagedCreate> VirtualMachineScaleSetUnmanagedDataDisk.Definition.IWithFromImageDiskSettings<VirtualMachineScaleSet.Definition.IWithUnmanagedCreate>.WithCaching(CachingTypes cachingType)
        {
            return this.WithCaching(cachingType);
        }

        /// <summary>
        /// Specifies the logical unit number for the unmanaged data disk.
        /// </summary>
        /// <param name="lun">The logical unit number.</param>
        /// <return>The next stage of unmanaged data disk definition.</return>
        VirtualMachineScaleSetUnmanagedDataDisk.Definition.IWithNewVhdDiskSettings<VirtualMachineScaleSet.Definition.IWithUnmanagedCreate> VirtualMachineScaleSetUnmanagedDataDisk.Definition.IWithNewVhdDiskSettings<VirtualMachineScaleSet.Definition.IWithUnmanagedCreate>.WithLun(int lun)
        {
            return this.WithLun(lun);
        }

        /// <summary>
        /// Specifies the logical unit number for the unmanaged data disk.
        /// </summary>
        /// <param name="lun">The logical unit number.</param>
        /// <return>The next stage of unmanaged data disk definition.</return>
        VirtualMachineScaleSetUnmanagedDataDisk.UpdateDefinition.IWithNewVhdDiskSettings<VirtualMachineScaleSet.Update.IWithApply> VirtualMachineScaleSetUnmanagedDataDisk.UpdateDefinition.IWithNewVhdDiskSettings<VirtualMachineScaleSet.Update.IWithApply>.WithLun(int lun)
        {
            return this.WithLun(lun);
        }

        /// <summary>
        /// Specifies the new logical unit number for the unmanaged data disk.
        /// </summary>
        /// <param name="lun">The logical unit number.</param>
        /// <return>The next stage of unmanaged data disk update.</return>
        VirtualMachineScaleSetUnmanagedDataDisk.Update.IUpdate VirtualMachineScaleSetUnmanagedDataDisk.Update.IWithDiskLun.WithLun(int lun)
        {
            return this.WithLun(lun);
        }

        /// <summary>
        /// Specifies that unmanaged disk needs to be created with a new VHD of given size.
        /// </summary>
        /// <param name="sizeInGB">The initial disk size in GB.</param>
        /// <return>The next stage of unmanaged data disk definition.</return>
        VirtualMachineScaleSetUnmanagedDataDisk.Definition.IWithNewVhdDiskSettings<VirtualMachineScaleSet.Definition.IWithUnmanagedCreate> VirtualMachineScaleSetUnmanagedDataDisk.Definition.IWithDiskSource<VirtualMachineScaleSet.Definition.IWithUnmanagedCreate>.WithNewVhd(int sizeInGB)
        {
            return this.WithNewVhd(sizeInGB);
        }

        /// <summary>
        /// Specifies that unmanaged disk needs to be created with a new VHD of given size.
        /// </summary>
        /// <param name="sizeInGB">The initial disk size in GB.</param>
        /// <return>The next stage of unmanaged data disk definition.</return>
        VirtualMachineScaleSetUnmanagedDataDisk.UpdateDefinition.IWithNewVhdDiskSettings<VirtualMachineScaleSet.Update.IWithApply> VirtualMachineScaleSetUnmanagedDataDisk.UpdateDefinition.IWithDiskSource<VirtualMachineScaleSet.Update.IWithApply>.WithNewVhd(int sizeInGB)
        {
            return this.WithNewVhd(sizeInGB);
        }

        /// <summary>
        /// Specifies the new size in GB for data disk.
        /// </summary>
        /// <param name="sizeInGB">The disk size in GB.</param>
        /// <return>The next stage of unmanaged data disk update.</return>
        VirtualMachineScaleSetUnmanagedDataDisk.Update.IUpdate VirtualMachineScaleSetUnmanagedDataDisk.Update.IWithDiskSize.WithSizeInGB(int sizeInGB)
        {
            return this.WithSizeInGB(sizeInGB);
        }

        /// <summary>
        /// Specifies the size in GB the unmanaged disk needs to be resized.
        /// </summary>
        /// <param name="sizeInGB">The disk size in GB.</param>
        /// <return>The next stage of unmanaged data disk definition.</return>
        VirtualMachineScaleSetUnmanagedDataDisk.Definition.IWithFromImageDiskSettings<VirtualMachineScaleSet.Definition.IWithUnmanagedCreate> VirtualMachineScaleSetUnmanagedDataDisk.Definition.IWithFromImageDiskSettings<VirtualMachineScaleSet.Definition.IWithUnmanagedCreate>.WithSizeInGB(int sizeInGB)
        {
            return this.WithSizeInGB(sizeInGB);
        }
    }
}