// <auto-generated>
// Copyright (c) Microsoft Corporation. All rights reserved.
// Licensed under the MIT License. See License.txt in the project root for
// license information.
//
// Code generated by Microsoft (R) AutoRest Code Generator.
// </auto-generated>

namespace Microsoft.Azure.Management.ContainerRegistry.Fluent.Models
{
    using Microsoft.Rest;
    using Microsoft.Rest.Azure;
    using Microsoft.Rest.Serialization;
    using Newtonsoft.Json;
    using System.Collections;
    using System.Collections.Generic;
    using System.Linq;

    /// <summary>
    /// The task that has the ARM resource and task properties.
    /// The  task will have all information to schedule a run against it.
    /// </summary>
    [Rest.Serialization.JsonTransformation]
    public partial class TaskInner : Microsoft.Azure.Management.ResourceManager.Fluent.Resource
    {
        /// <summary>
        /// Initializes a new instance of the TaskInner class.
        /// </summary>
        public TaskInner()
        {
            CustomInit();
        }

        /// <summary>
        /// Initializes a new instance of the TaskInner class.
        /// </summary>
        /// <param name="platform">The platform properties against which the
        /// run has to happen.</param>
        /// <param name="step">The properties of a task step.</param>
        /// <param name="provisioningState">The provisioning state of the task.
        /// Possible values include: 'Creating', 'Updating', 'Deleting',
        /// 'Succeeded', 'Failed', 'Canceled'</param>
        /// <param name="creationDate">The creation date of task.</param>
        /// <param name="status">The current status of task. Possible values
        /// include: 'Disabled', 'Enabled'</param>
        /// <param name="agentConfiguration">The machine configuration of the
        /// run agent.</param>
        /// <param name="timeout">Run timeout in seconds.</param>
        /// <param name="trigger">The properties that describe all triggers for
        /// the task.</param>
        public TaskInner(PlatformProperties platform, TaskStepProperties step, string location = default(string), string id = default(string), string name = default(string), string type = default(string), IDictionary<string, string> tags = default(IDictionary<string, string>), string provisioningState = default(string), System.DateTime? creationDate = default(System.DateTime?), string status = default(string), AgentProperties agentConfiguration = default(AgentProperties), int? timeout = default(int?), TriggerProperties trigger = default(TriggerProperties))
            : base(location, id, name, type, tags)
        {
            ProvisioningState = provisioningState;
            CreationDate = creationDate;
            Status = status;
            Platform = platform;
            AgentConfiguration = agentConfiguration;
            Timeout = timeout;
            Step = step;
            Trigger = trigger;
            CustomInit();
        }

        /// <summary>
        /// An initialization method that performs custom operations like setting defaults
        /// </summary>
        partial void CustomInit();

        /// <summary>
        /// Gets the provisioning state of the task. Possible values include:
        /// 'Creating', 'Updating', 'Deleting', 'Succeeded', 'Failed',
        /// 'Canceled'
        /// </summary>
        [JsonProperty(PropertyName = "properties.provisioningState")]
        public string ProvisioningState { get; private set; }

        /// <summary>
        /// Gets the creation date of task.
        /// </summary>
        [JsonProperty(PropertyName = "properties.creationDate")]
        public System.DateTime? CreationDate { get; private set; }

        /// <summary>
        /// Gets or sets the current status of task. Possible values include:
        /// 'Disabled', 'Enabled'
        /// </summary>
        [JsonProperty(PropertyName = "properties.status")]
        public string Status { get; set; }

        /// <summary>
        /// Gets or sets the platform properties against which the run has to
        /// happen.
        /// </summary>
        [JsonProperty(PropertyName = "properties.platform")]
        public PlatformProperties Platform { get; set; }

        /// <summary>
        /// Gets or sets the machine configuration of the run agent.
        /// </summary>
        [JsonProperty(PropertyName = "properties.agentConfiguration")]
        public AgentProperties AgentConfiguration { get; set; }

        /// <summary>
        /// Gets or sets run timeout in seconds.
        /// </summary>
        [JsonProperty(PropertyName = "properties.timeout")]
        public int? Timeout { get; set; }

        /// <summary>
        /// Gets or sets the properties of a task step.
        /// </summary>
        [JsonProperty(PropertyName = "properties.step")]
        public TaskStepProperties Step { get; set; }

        /// <summary>
        /// Gets or sets the properties that describe all triggers for the
        /// task.
        /// </summary>
        [JsonProperty(PropertyName = "properties.trigger")]
        public TriggerProperties Trigger { get; set; }

        /// <summary>
        /// Validate the object.
        /// </summary>
        /// <exception cref="ValidationException">
        /// Thrown if validation fails
        /// </exception>
        public virtual void Validate()
        {
            if (Platform == null)
            {
                throw new ValidationException(ValidationRules.CannotBeNull, "Platform");
            }
            if (Step == null)
            {
                throw new ValidationException(ValidationRules.CannotBeNull, "Step");
            }
            if (Platform != null)
            {
                Platform.Validate();
            }
            if (Timeout > 28800)
            {
                throw new ValidationException(ValidationRules.InclusiveMaximum, "Timeout", 28800);
            }
            if (Timeout < 300)
            {
                throw new ValidationException(ValidationRules.InclusiveMinimum, "Timeout", 300);
            }
            if (Trigger != null)
            {
                Trigger.Validate();
            }
        }
    }
}
