// <auto-generated>
// Copyright (c) Microsoft Corporation. All rights reserved.
// Licensed under the MIT License. See License.txt in the project root for
// license information.
//
// Code generated by Microsoft (R) AutoRest Code Generator.
// </auto-generated>

namespace Microsoft.Azure.Management.ContainerService.Fluent.Models
{
    using Microsoft.Azure.Management.ResourceManager;
    using Microsoft.Azure.Management.ResourceManager.Fluent;
    using Microsoft.Rest;
    using Microsoft.Rest.Serialization;
    using Newtonsoft.Json;
    using System.Collections;
    using System.Collections.Generic;
    using System.Linq;

    /// <summary>
    /// Container service.
    /// </summary>
    [Rest.Serialization.JsonTransformation]
    public partial class ContainerServiceInner : Management.ResourceManager.Fluent.Resource
    {
        /// <summary>
        /// Initializes a new instance of the ContainerServiceInner class.
        /// </summary>
        public ContainerServiceInner()
        {
            CustomInit();
        }

        /// <summary>
        /// Initializes a new instance of the ContainerServiceInner class.
        /// </summary>
        /// <param name="orchestratorProfile">Profile for the container service
        /// orchestrator.</param>
        /// <param name="masterProfile">Profile for the container service
        /// master.</param>
        /// <param name="linuxProfile">Profile for Linux VMs in the container
        /// service cluster.</param>
        /// <param name="provisioningState">The current deployment or
        /// provisioning state, which only appears in the response.</param>
        /// <param name="customProfile">Properties to configure a custom
        /// container service cluster.</param>
        /// <param name="servicePrincipalProfile">Information about a service
        /// principal identity for the cluster to use for manipulating Azure
        /// APIs. Exact one of secret or keyVaultSecretRef need to be
        /// specified.</param>
        /// <param name="agentPoolProfiles">Properties of the agent
        /// pool.</param>
        /// <param name="windowsProfile">Profile for Windows VMs in the
        /// container service cluster.</param>
        /// <param name="diagnosticsProfile">Profile for diagnostics in the
        /// container service cluster.</param>
        public ContainerServiceInner(string location, ContainerServiceOrchestratorProfile orchestratorProfile, ContainerServiceMasterProfile masterProfile, ContainerServiceLinuxProfile linuxProfile, string id = default(string), string name = default(string), string type = default(string), IDictionary<string, string> tags = default(IDictionary<string, string>), string provisioningState = default(string), ContainerServiceCustomProfile customProfile = default(ContainerServiceCustomProfile), ContainerServiceServicePrincipalProfile servicePrincipalProfile = default(ContainerServiceServicePrincipalProfile), IList<ContainerServiceAgentPoolProfile> agentPoolProfiles = default(IList<ContainerServiceAgentPoolProfile>), ContainerServiceWindowsProfile windowsProfile = default(ContainerServiceWindowsProfile), ContainerServiceDiagnosticsProfile diagnosticsProfile = default(ContainerServiceDiagnosticsProfile))
            : base(location, id, name, type, tags)
        {
            ProvisioningState = provisioningState;
            OrchestratorProfile = orchestratorProfile;
            CustomProfile = customProfile;
            ServicePrincipalProfile = servicePrincipalProfile;
            MasterProfile = masterProfile;
            AgentPoolProfiles = agentPoolProfiles;
            WindowsProfile = windowsProfile;
            LinuxProfile = linuxProfile;
            DiagnosticsProfile = diagnosticsProfile;
            CustomInit();
        }

        /// <summary>
        /// An initialization method that performs custom operations like setting defaults
        /// </summary>
        partial void CustomInit();

        /// <summary>
        /// Gets the current deployment or provisioning state, which only
        /// appears in the response.
        /// </summary>
        [JsonProperty(PropertyName = "properties.provisioningState")]
        public string ProvisioningState { get; private set; }

        /// <summary>
        /// Gets or sets profile for the container service orchestrator.
        /// </summary>
        [JsonProperty(PropertyName = "properties.orchestratorProfile")]
        public ContainerServiceOrchestratorProfile OrchestratorProfile { get; set; }

        /// <summary>
        /// Gets or sets properties to configure a custom container service
        /// cluster.
        /// </summary>
        [JsonProperty(PropertyName = "properties.customProfile")]
        public ContainerServiceCustomProfile CustomProfile { get; set; }

        /// <summary>
        /// Gets or sets information about a service principal identity for the
        /// cluster to use for manipulating Azure APIs. Exact one of secret or
        /// keyVaultSecretRef need to be specified.
        /// </summary>
        [JsonProperty(PropertyName = "properties.servicePrincipalProfile")]
        public ContainerServiceServicePrincipalProfile ServicePrincipalProfile { get; set; }

        /// <summary>
        /// Gets or sets profile for the container service master.
        /// </summary>
        [JsonProperty(PropertyName = "properties.masterProfile")]
        public ContainerServiceMasterProfile MasterProfile { get; set; }

        /// <summary>
        /// Gets or sets properties of the agent pool.
        /// </summary>
        [JsonProperty(PropertyName = "properties.agentPoolProfiles")]
        public IList<ContainerServiceAgentPoolProfile> AgentPoolProfiles { get; set; }

        /// <summary>
        /// Gets or sets profile for Windows VMs in the container service
        /// cluster.
        /// </summary>
        [JsonProperty(PropertyName = "properties.windowsProfile")]
        public ContainerServiceWindowsProfile WindowsProfile { get; set; }

        /// <summary>
        /// Gets or sets profile for Linux VMs in the container service
        /// cluster.
        /// </summary>
        [JsonProperty(PropertyName = "properties.linuxProfile")]
        public ContainerServiceLinuxProfile LinuxProfile { get; set; }

        /// <summary>
        /// Gets or sets profile for diagnostics in the container service
        /// cluster.
        /// </summary>
        [JsonProperty(PropertyName = "properties.diagnosticsProfile")]
        public ContainerServiceDiagnosticsProfile DiagnosticsProfile { get; set; }

        /// <summary>
        /// Validate the object.
        /// </summary>
        /// <exception cref="ValidationException">
        /// Thrown if validation fails
        /// </exception>
        public override void Validate()
        {
            base.Validate();
            if (OrchestratorProfile == null)
            {
                throw new ValidationException(ValidationRules.CannotBeNull, "OrchestratorProfile");
            }
            if (MasterProfile == null)
            {
                throw new ValidationException(ValidationRules.CannotBeNull, "MasterProfile");
            }
            if (LinuxProfile == null)
            {
                throw new ValidationException(ValidationRules.CannotBeNull, "LinuxProfile");
            }
            if (OrchestratorProfile != null)
            {
                OrchestratorProfile.Validate();
            }
            if (CustomProfile != null)
            {
                CustomProfile.Validate();
            }
            if (ServicePrincipalProfile != null)
            {
                ServicePrincipalProfile.Validate();
            }
            if (MasterProfile != null)
            {
                MasterProfile.Validate();
            }
            if (AgentPoolProfiles != null)
            {
                foreach (var element in AgentPoolProfiles)
                {
                    if (element != null)
                    {
                        element.Validate();
                    }
                }
            }
            if (WindowsProfile != null)
            {
                WindowsProfile.Validate();
            }
            if (LinuxProfile != null)
            {
                LinuxProfile.Validate();
            }
            if (DiagnosticsProfile != null)
            {
                DiagnosticsProfile.Validate();
            }
        }
    }
}
