// <auto-generated>
// Copyright (c) Microsoft Corporation. All rights reserved.
// Licensed under the MIT License. See License.txt in the project root for
// license information.
//
// Code generated by Microsoft (R) AutoRest Code Generator.
// </auto-generated>

namespace Microsoft.Azure.Management.Network.Fluent.Models
{
    using Microsoft.Azure.Management.ResourceManager;
    using Microsoft.Azure.Management.ResourceManager.Fluent;
    using Microsoft.Rest;
    using Microsoft.Rest.Serialization;
    using Newtonsoft.Json;
    using System.Collections;
    using System.Collections.Generic;
    using System.Linq;

    /// <summary>
    /// A load balancer probe.
    /// </summary>
    [Rest.Serialization.JsonTransformation]
    public partial class ProbeInner : Management.ResourceManager.Fluent.SubResource
    {
        /// <summary>
        /// Initializes a new instance of the ProbeInner class.
        /// </summary>
        public ProbeInner()
        {
            CustomInit();
        }

        /// <summary>
        /// Initializes a new instance of the ProbeInner class.
        /// </summary>
        /// <param name="protocol">The protocol of the end point. If 'Tcp' is
        /// specified, a received ACK is required for the probe to be
        /// successful. If 'Http' or 'Https' is specified, a 200 OK response
        /// from the specifies URI is required for the probe to be successful.
        /// Possible values include: 'Http', 'Tcp', 'Https'</param>
        /// <param name="port">The port for communicating the probe. Possible
        /// values range from 1 to 65535, inclusive.</param>
        /// <param name="loadBalancingRules">The load balancer rules that use
        /// this probe.</param>
        /// <param name="intervalInSeconds">The interval, in seconds, for how
        /// frequently to probe the endpoint for health status. Typically, the
        /// interval is slightly less than half the allocated timeout period
        /// (in seconds) which allows two full probes before taking the
        /// instance out of rotation. The default value is 15, the minimum
        /// value is 5.</param>
        /// <param name="numberOfProbes">The number of probes where if no
        /// response, will result in stopping further traffic from being
        /// delivered to the endpoint. This values allows endpoints to be taken
        /// out of rotation faster or slower than the typical times used in
        /// Azure.</param>
        /// <param name="requestPath">The URI used for requesting health status
        /// from the VM. Path is required if a protocol is set to http.
        /// Otherwise, it is not allowed. There is no default value.</param>
        /// <param name="provisioningState">The provisioning state of the probe
        /// resource. Possible values include: 'Succeeded', 'Updating',
        /// 'Deleting', 'Failed'</param>
        /// <param name="name">The name of the resource that is unique within
        /// the set of probes used by the load balancer. This name can be used
        /// to access the resource.</param>
        /// <param name="etag">A unique read-only string that changes whenever
        /// the resource is updated.</param>
        /// <param name="type">Type of the resource.</param>
        public ProbeInner(ProbeProtocol protocol, int port, string id = default(string), IList<Management.ResourceManager.Fluent.SubResource> loadBalancingRules = default(IList<Management.ResourceManager.Fluent.SubResource>), int? intervalInSeconds = default(int?), int? numberOfProbes = default(int?), string requestPath = default(string), ProvisioningState provisioningState = default(ProvisioningState), string name = default(string), string etag = default(string), string type = default(string))
            : base(id)
        {
            LoadBalancingRules = loadBalancingRules;
            Protocol = protocol;
            Port = port;
            IntervalInSeconds = intervalInSeconds;
            NumberOfProbes = numberOfProbes;
            RequestPath = requestPath;
            ProvisioningState = provisioningState;
            Name = name;
            Etag = etag;
            Type = type;
            CustomInit();
        }

        /// <summary>
        /// An initialization method that performs custom operations like setting defaults
        /// </summary>
        partial void CustomInit();

        /// <summary>
        /// Gets the load balancer rules that use this probe.
        /// </summary>
        [JsonProperty(PropertyName = "properties.loadBalancingRules")]
        public IList<Management.ResourceManager.Fluent.SubResource> LoadBalancingRules { get; private set; }

        /// <summary>
        /// Gets or sets the protocol of the end point. If 'Tcp' is specified,
        /// a received ACK is required for the probe to be successful. If
        /// 'Http' or 'Https' is specified, a 200 OK response from the
        /// specifies URI is required for the probe to be successful. Possible
        /// values include: 'Http', 'Tcp', 'Https'
        /// </summary>
        [JsonProperty(PropertyName = "properties.protocol")]
        public ProbeProtocol Protocol { get; set; }

        /// <summary>
        /// Gets or sets the port for communicating the probe. Possible values
        /// range from 1 to 65535, inclusive.
        /// </summary>
        [JsonProperty(PropertyName = "properties.port")]
        public int Port { get; set; }

        /// <summary>
        /// Gets or sets the interval, in seconds, for how frequently to probe
        /// the endpoint for health status. Typically, the interval is slightly
        /// less than half the allocated timeout period (in seconds) which
        /// allows two full probes before taking the instance out of rotation.
        /// The default value is 15, the minimum value is 5.
        /// </summary>
        [JsonProperty(PropertyName = "properties.intervalInSeconds")]
        public int? IntervalInSeconds { get; set; }

        /// <summary>
        /// Gets or sets the number of probes where if no response, will result
        /// in stopping further traffic from being delivered to the endpoint.
        /// This values allows endpoints to be taken out of rotation faster or
        /// slower than the typical times used in Azure.
        /// </summary>
        [JsonProperty(PropertyName = "properties.numberOfProbes")]
        public int? NumberOfProbes { get; set; }

        /// <summary>
        /// Gets or sets the URI used for requesting health status from the VM.
        /// Path is required if a protocol is set to http. Otherwise, it is not
        /// allowed. There is no default value.
        /// </summary>
        [JsonProperty(PropertyName = "properties.requestPath")]
        public string RequestPath { get; set; }

        /// <summary>
        /// Gets the provisioning state of the probe resource. Possible values
        /// include: 'Succeeded', 'Updating', 'Deleting', 'Failed'
        /// </summary>
        [JsonProperty(PropertyName = "properties.provisioningState")]
        public ProvisioningState ProvisioningState { get; private set; }

        /// <summary>
        /// Gets or sets the name of the resource that is unique within the set
        /// of probes used by the load balancer. This name can be used to
        /// access the resource.
        /// </summary>
        [JsonProperty(PropertyName = "name")]
        public string Name { get; set; }

        /// <summary>
        /// Gets a unique read-only string that changes whenever the resource
        /// is updated.
        /// </summary>
        [JsonProperty(PropertyName = "etag")]
        public string Etag { get; private set; }

        /// <summary>
        /// Gets type of the resource.
        /// </summary>
        [JsonProperty(PropertyName = "type")]
        public string Type { get; private set; }

        /// <summary>
        /// Validate the object.
        /// </summary>
        /// <exception cref="ValidationException">
        /// Thrown if validation fails
        /// </exception>
        public virtual void Validate()
        {
            if (Protocol == null)
            {
                throw new ValidationException(ValidationRules.CannotBeNull, "Protocol");
            }
        }
    }
}
