// <auto-generated>
// Copyright (c) Microsoft Corporation. All rights reserved.
// Licensed under the MIT License. See License.txt in the project root for
// license information.
//
// Code generated by Microsoft (R) AutoRest Code Generator.
// </auto-generated>

namespace Microsoft.Azure.Management.Network.Fluent.Models
{
    using Microsoft.Rest;
    using Newtonsoft.Json;
    using System.Collections;
    using System.Collections.Generic;
    using System.Linq;

    /// <summary>
    /// Defines contents of a web application rule.
    /// </summary>
    public partial class WebApplicationFirewallCustomRule
    {
        /// <summary>
        /// Initializes a new instance of the WebApplicationFirewallCustomRule
        /// class.
        /// </summary>
        public WebApplicationFirewallCustomRule()
        {
            CustomInit();
        }

        /// <summary>
        /// Initializes a new instance of the WebApplicationFirewallCustomRule
        /// class.
        /// </summary>
        /// <param name="priority">Describes priority of the rule. Rules with a
        /// lower value will be evaluated before rules with a higher
        /// value.</param>
        /// <param name="ruleType">Describes type of rule. Possible values
        /// include: 'MatchRule', 'Invalid'</param>
        /// <param name="matchConditions">List of match conditions.</param>
        /// <param name="action">Type of Actions. Possible values include:
        /// 'Allow', 'Block', 'Log'</param>
        /// <param name="name">The name of the resource that is unique within a
        /// policy. This name can be used to access the resource.</param>
        /// <param name="etag">A unique read-only string that changes whenever
        /// the resource is updated.</param>
        public WebApplicationFirewallCustomRule(int priority, WebApplicationFirewallRuleType ruleType, IList<MatchCondition> matchConditions, WebApplicationFirewallAction action, string name = default(string), string etag = default(string))
        {
            Name = name;
            Etag = etag;
            Priority = priority;
            RuleType = ruleType;
            MatchConditions = matchConditions;
            Action = action;
            CustomInit();
        }

        /// <summary>
        /// An initialization method that performs custom operations like setting defaults
        /// </summary>
        partial void CustomInit();

        /// <summary>
        /// Gets or sets the name of the resource that is unique within a
        /// policy. This name can be used to access the resource.
        /// </summary>
        [JsonProperty(PropertyName = "name")]
        public string Name { get; set; }

        /// <summary>
        /// Gets a unique read-only string that changes whenever the resource
        /// is updated.
        /// </summary>
        [JsonProperty(PropertyName = "etag")]
        public string Etag { get; private set; }

        /// <summary>
        /// Gets or sets describes priority of the rule. Rules with a lower
        /// value will be evaluated before rules with a higher value.
        /// </summary>
        [JsonProperty(PropertyName = "priority")]
        public int Priority { get; set; }

        /// <summary>
        /// Gets or sets describes type of rule. Possible values include:
        /// 'MatchRule', 'Invalid'
        /// </summary>
        [JsonProperty(PropertyName = "ruleType")]
        public WebApplicationFirewallRuleType RuleType { get; set; }

        /// <summary>
        /// Gets or sets list of match conditions.
        /// </summary>
        [JsonProperty(PropertyName = "matchConditions")]
        public IList<MatchCondition> MatchConditions { get; set; }

        /// <summary>
        /// Gets or sets type of Actions. Possible values include: 'Allow',
        /// 'Block', 'Log'
        /// </summary>
        [JsonProperty(PropertyName = "action")]
        public WebApplicationFirewallAction Action { get; set; }

        /// <summary>
        /// Validate the object.
        /// </summary>
        /// <exception cref="ValidationException">
        /// Thrown if validation fails
        /// </exception>
        public virtual void Validate()
        {
            if (RuleType == null)
            {
                throw new ValidationException(ValidationRules.CannotBeNull, "RuleType");
            }
            if (MatchConditions == null)
            {
                throw new ValidationException(ValidationRules.CannotBeNull, "MatchConditions");
            }
            if (Action == null)
            {
                throw new ValidationException(ValidationRules.CannotBeNull, "Action");
            }
            if (Name != null)
            {
                if (Name.Length > 128)
                {
                    throw new ValidationException(ValidationRules.MaxLength, "Name", 128);
                }
            }
            if (MatchConditions != null)
            {
                foreach (var element in MatchConditions)
                {
                    if (element != null)
                    {
                        element.Validate();
                    }
                }
            }
        }
    }
}
