﻿// Copyright (c) Microsoft Corporation. All rights reserved.
// Licensed under the MIT License. See License.txt in the project root for license information.

namespace Microsoft.Azure.Management.ResourceManager.Fluent.Core
{
    /// <summary>
    /// Defines values for country codes for phone number prefixes.
    /// E.g.the country code for +1 123 456 7890 is '+1'.
    /// </summary>
    public class CountryPhoneCode : ExpandableStringEnum<CountryPhoneCode>
    {
        public static readonly CountryPhoneCode Albania = Parse("+355");
        public static readonly CountryPhoneCode Algeria = Parse("+213");
        public static readonly CountryPhoneCode Andorra = Parse("+376");
        public static readonly CountryPhoneCode Angola = Parse("+244");
        public static readonly CountryPhoneCode Antarctica = Parse("+672");
        public static readonly CountryPhoneCode AntiguaAndBarbuda = Parse("+1");
        public static readonly CountryPhoneCode Argentina = Parse("+54");
        public static readonly CountryPhoneCode Armenia = Parse("+374");
        public static readonly CountryPhoneCode Aruba = Parse("+297");
        public static readonly CountryPhoneCode AscensionIsland = Parse("+247");
        public static readonly CountryPhoneCode Australia = Parse("+61");
        public static readonly CountryPhoneCode Austria = Parse("+43");
        public static readonly CountryPhoneCode Azerbaijan = Parse("+994");
        public static readonly CountryPhoneCode Bahamas = Parse("+1");
        public static readonly CountryPhoneCode Bahrain = Parse("+973");
        public static readonly CountryPhoneCode Bangladesh = Parse("+880");
        public static readonly CountryPhoneCode Barbados = Parse("+1");
        public static readonly CountryPhoneCode Belarus = Parse("+375");
        public static readonly CountryPhoneCode Belgium = Parse("+32");
        public static readonly CountryPhoneCode Belize = Parse("+501");
        public static readonly CountryPhoneCode Benin = Parse("+229");
        public static readonly CountryPhoneCode Bermuda = Parse("+1");
        public static readonly CountryPhoneCode Bhutan = Parse("+975");
        public static readonly CountryPhoneCode Bolivia = Parse("+591");
        public static readonly CountryPhoneCode BosniaAndHerzegovina = Parse("+387");
        public static readonly CountryPhoneCode Botswana = Parse("+267");
        public static readonly CountryPhoneCode BouvetIsland = Parse("+47");
        public static readonly CountryPhoneCode Brazil = Parse("+55");
        public static readonly CountryPhoneCode BritishIndianOceanTerritory = Parse("+44");
        public static readonly CountryPhoneCode Brunei = Parse("+673");
        public static readonly CountryPhoneCode Bulgaria = Parse("+359");
        public static readonly CountryPhoneCode BurkinaFaso = Parse("+226");
        public static readonly CountryPhoneCode Burundi = Parse("+257");
        public static readonly CountryPhoneCode CaboVerde = Parse("+238");
        public static readonly CountryPhoneCode Cambodia = Parse("+855");
        public static readonly CountryPhoneCode Cameroon = Parse("+237");
        public static readonly CountryPhoneCode Canada = Parse("+1");
        public static readonly CountryPhoneCode CaymanIslands = Parse("+1");
        public static readonly CountryPhoneCode CentralAfricanRepublic = Parse("+236");
        public static readonly CountryPhoneCode Chad = Parse("+235");
        public static readonly CountryPhoneCode Chile = Parse("+56");
        public static readonly CountryPhoneCode China = Parse("+86");
        public static readonly CountryPhoneCode ChristmasIsland = Parse("+61");
        public static readonly CountryPhoneCode CocosKeelingIslands = Parse("+61");
        public static readonly CountryPhoneCode Colombia = Parse("+57");
        public static readonly CountryPhoneCode Comoros = Parse("+269");
        public static readonly CountryPhoneCode Congo = Parse("+242");
        public static readonly CountryPhoneCode CongoDrc = Parse("+243");
        public static readonly CountryPhoneCode CookIslands = Parse("+682");
        public static readonly CountryPhoneCode CostaRica = Parse("+506");
        public static readonly CountryPhoneCode Croatia = Parse("+385");
        public static readonly CountryPhoneCode Cuba = Parse("+53");
        public static readonly CountryPhoneCode Cyprus = Parse("+357");
        public static readonly CountryPhoneCode CzechRepublic = Parse("+420");
        public static readonly CountryPhoneCode Denmark = Parse("+45");
        public static readonly CountryPhoneCode Djibouti = Parse("+253");
        public static readonly CountryPhoneCode Dominica = Parse("+1");
        public static readonly CountryPhoneCode DominicanRepublic = Parse("+1");
        public static readonly CountryPhoneCode Ecuador = Parse("+593");
        public static readonly CountryPhoneCode Egypt = Parse("+20");
        public static readonly CountryPhoneCode ElSalvador = Parse("+503");
        public static readonly CountryPhoneCode EquatorialGuinea = Parse("+240");
        public static readonly CountryPhoneCode Eritrea = Parse("+291");
        public static readonly CountryPhoneCode Estonia = Parse("+372");
        public static readonly CountryPhoneCode Ethiopia = Parse("+251");
        public static readonly CountryPhoneCode FalklandIslands = Parse("+500");
        public static readonly CountryPhoneCode FaroeIslands = Parse("+298");
        public static readonly CountryPhoneCode FijiIslands = Parse("+679");
        public static readonly CountryPhoneCode Finland = Parse("+358");
        public static readonly CountryPhoneCode France = Parse("+33");
        public static readonly CountryPhoneCode FrenchGuiana = Parse("+594");
        public static readonly CountryPhoneCode FrenchPolynesia = Parse("+689");
        public static readonly CountryPhoneCode Gabon = Parse("+241");
        public static readonly CountryPhoneCode GambiaThe = Parse("+220");
        public static readonly CountryPhoneCode Georgia = Parse("+995");
        public static readonly CountryPhoneCode Germany = Parse("+49");
        public static readonly CountryPhoneCode Ghana = Parse("+233");
        public static readonly CountryPhoneCode Gibraltar = Parse("+350");
        public static readonly CountryPhoneCode Greece = Parse("+30");
        public static readonly CountryPhoneCode Greenland = Parse("+299");
        public static readonly CountryPhoneCode Grenada = Parse("+1");
        public static readonly CountryPhoneCode Guadeloupe = Parse("+590");
        public static readonly CountryPhoneCode Guam = Parse("+1");
        public static readonly CountryPhoneCode Guatemala = Parse("+502");
        public static readonly CountryPhoneCode Guernsey = Parse("+44");
        public static readonly CountryPhoneCode Guinea = Parse("+224");
        public static readonly CountryPhoneCode GuineaBissau = Parse("+245");
        public static readonly CountryPhoneCode Guyana = Parse("+592");
        public static readonly CountryPhoneCode Haiti = Parse("+509");
        public static readonly CountryPhoneCode HolySeeVaticanCity = Parse("+379");
        public static readonly CountryPhoneCode Honduras = Parse("+504");
        public static readonly CountryPhoneCode HongKongSar = Parse("+852");
        public static readonly CountryPhoneCode Hungary = Parse("+36");
        public static readonly CountryPhoneCode Iceland = Parse("+354");
        public static readonly CountryPhoneCode India = Parse("+91");
        public static readonly CountryPhoneCode Indonesia = Parse("+62");
        public static readonly CountryPhoneCode Iran = Parse("+98");
        public static readonly CountryPhoneCode Iraq = Parse("+964");
        public static readonly CountryPhoneCode Ireland = Parse("+353");
        public static readonly CountryPhoneCode IsleOfMan = Parse("+44");
        public static readonly CountryPhoneCode Israel = Parse("+972");
        public static readonly CountryPhoneCode Italy = Parse("+39");
        public static readonly CountryPhoneCode Jamaica = Parse("+1");
        public static readonly CountryPhoneCode JanMayen = Parse("+47");
        public static readonly CountryPhoneCode Japan = Parse("+81");
        public static readonly CountryPhoneCode Jersey = Parse("+44");
        public static readonly CountryPhoneCode Jordan = Parse("+962");
        public static readonly CountryPhoneCode Kazakhstan = Parse("+7");
        public static readonly CountryPhoneCode Kenya = Parse("+254");
        public static readonly CountryPhoneCode Kiribati = Parse("+686");
        public static readonly CountryPhoneCode Korea = Parse("+82");
        public static readonly CountryPhoneCode Kuwait = Parse("+965");
        public static readonly CountryPhoneCode Kyrgyzstan = Parse("+996");
        public static readonly CountryPhoneCode Laos = Parse("+856");
        public static readonly CountryPhoneCode Latvia = Parse("+371");
        public static readonly CountryPhoneCode Lebanon = Parse("+961");
        public static readonly CountryPhoneCode Lesotho = Parse("+266");
        public static readonly CountryPhoneCode Liberia = Parse("+231");
        public static readonly CountryPhoneCode Libya = Parse("+218");
        public static readonly CountryPhoneCode Liechtenstein = Parse("+423");
        public static readonly CountryPhoneCode Lithuania = Parse("+370");
        public static readonly CountryPhoneCode Luxembourg = Parse("+352");
        public static readonly CountryPhoneCode MacaoSar = Parse("+853");
        public static readonly CountryPhoneCode Macedonia = Parse("+389");
        public static readonly CountryPhoneCode Madagascar = Parse("+261");
        public static readonly CountryPhoneCode Malawi = Parse("+265");
        public static readonly CountryPhoneCode Malaysia = Parse("+60");
        public static readonly CountryPhoneCode Maldives = Parse("+960");
        public static readonly CountryPhoneCode Mali = Parse("+223");
        public static readonly CountryPhoneCode Malta = Parse("+356");
        public static readonly CountryPhoneCode MarshallIslands = Parse("+692");
        public static readonly CountryPhoneCode Martinique = Parse("+596");
        public static readonly CountryPhoneCode Mauritania = Parse("+222");
        public static readonly CountryPhoneCode Mauritius = Parse("+230");
        public static readonly CountryPhoneCode Mayotte = Parse("+262");
        public static readonly CountryPhoneCode Mexico = Parse("+52");
        public static readonly CountryPhoneCode Micronesia = Parse("+691");
        public static readonly CountryPhoneCode Moldova = Parse("+373");
        public static readonly CountryPhoneCode Monaco = Parse("+377");
        public static readonly CountryPhoneCode Mongolia = Parse("+976");
        public static readonly CountryPhoneCode Montenegro = Parse("+382");
        public static readonly CountryPhoneCode Montserrat = Parse("+1");
        public static readonly CountryPhoneCode Morocco = Parse("+212");
        public static readonly CountryPhoneCode Mozambique = Parse("+258");
        public static readonly CountryPhoneCode Myanmar = Parse("+95");
        public static readonly CountryPhoneCode Namibia = Parse("+264");
        public static readonly CountryPhoneCode Nauru = Parse("+674");
        public static readonly CountryPhoneCode Nepal = Parse("+977");
        public static readonly CountryPhoneCode Netherlands = Parse("+31");
        public static readonly CountryPhoneCode NetherlandsAntillesFormer = Parse("+599");
        public static readonly CountryPhoneCode NewCaledonia = Parse("+687");
        public static readonly CountryPhoneCode NewZealand = Parse("+64");
        public static readonly CountryPhoneCode Nicaragua = Parse("+505");
        public static readonly CountryPhoneCode Niger = Parse("+227");
        public static readonly CountryPhoneCode Nigeria = Parse("+234");
        public static readonly CountryPhoneCode Niue = Parse("+683");
        public static readonly CountryPhoneCode NorthKorea = Parse("+850");
        public static readonly CountryPhoneCode NorthernMarianaIslands = Parse("+1");
        public static readonly CountryPhoneCode Norway = Parse("+47");
        public static readonly CountryPhoneCode Oman = Parse("+968");
        public static readonly CountryPhoneCode Pakistan = Parse("+92");
        public static readonly CountryPhoneCode Palau = Parse("+680");
        public static readonly CountryPhoneCode PalestinianAuthority_970 = Parse("+970");
        public static readonly CountryPhoneCode PalestinianAuthority_972 = Parse("+972");
        public static readonly CountryPhoneCode Panama = Parse("+507");
        public static readonly CountryPhoneCode PapuaNewGuinea = Parse("+675");
        public static readonly CountryPhoneCode Paraguay = Parse("+595");
        public static readonly CountryPhoneCode Peru = Parse("+51");
        public static readonly CountryPhoneCode Philippines = Parse("+63");
        public static readonly CountryPhoneCode Poland = Parse("+48");
        public static readonly CountryPhoneCode Portugal = Parse("+351");
        public static readonly CountryPhoneCode Qatar = Parse("+974");
        public static readonly CountryPhoneCode IvoryCoast = Parse("+225");
        public static readonly CountryPhoneCode Reunion = Parse("+262");
        public static readonly CountryPhoneCode Romania = Parse("+40");
        public static readonly CountryPhoneCode Russia = Parse("+7");
        public static readonly CountryPhoneCode Rwanda = Parse("+250");
        public static readonly CountryPhoneCode SaintHelenaAscensionAndTristanDaCunha = Parse("+290");
        public static readonly CountryPhoneCode Samoa = Parse("+685");
        public static readonly CountryPhoneCode SanMarino = Parse("+378");
        public static readonly CountryPhoneCode SaoTomeAndPrincipe = Parse("+239");
        public static readonly CountryPhoneCode SaudiArabia = Parse("+966");
        public static readonly CountryPhoneCode Senegal = Parse("+221");
        public static readonly CountryPhoneCode Serbia = Parse("+381");
        public static readonly CountryPhoneCode Seychelles = Parse("+248");
        public static readonly CountryPhoneCode SierraLeone = Parse("+232");
        public static readonly CountryPhoneCode Singapore = Parse("+65");
        public static readonly CountryPhoneCode Slovakia = Parse("+421");
        public static readonly CountryPhoneCode Slovenia = Parse("+386");
        public static readonly CountryPhoneCode SolomonIslands = Parse("+677");
        public static readonly CountryPhoneCode Somalia = Parse("+252");
        public static readonly CountryPhoneCode SouthAfrica = Parse("+27");
        public static readonly CountryPhoneCode Spain = Parse("+34");
        public static readonly CountryPhoneCode SriLanka = Parse("+94");
        public static readonly CountryPhoneCode StKittsAndNevis = Parse("+1");
        public static readonly CountryPhoneCode StLucia = Parse("+1");
        public static readonly CountryPhoneCode StPierreAndMiquelon = Parse("+508");
        public static readonly CountryPhoneCode StVincentAndTheGrenadines = Parse("+1");
        public static readonly CountryPhoneCode Sudan = Parse("+249");
        public static readonly CountryPhoneCode Suriname = Parse("+597");
        public static readonly CountryPhoneCode Swaziland = Parse("+268");
        public static readonly CountryPhoneCode Sweden = Parse("+46");
        public static readonly CountryPhoneCode Switzerland = Parse("+41");
        public static readonly CountryPhoneCode Syria = Parse("+963");
        public static readonly CountryPhoneCode Taiwan = Parse("+886");
        public static readonly CountryPhoneCode Tajikistan = Parse("+992");
        public static readonly CountryPhoneCode Tanzania = Parse("+255");
        public static readonly CountryPhoneCode Thailand = Parse("+66");
        public static readonly CountryPhoneCode TimorLeste = Parse("+670");
        public static readonly CountryPhoneCode Togo = Parse("+228");
        public static readonly CountryPhoneCode Tokelau = Parse("+690");
        public static readonly CountryPhoneCode Tonga = Parse("+676");
        public static readonly CountryPhoneCode TrinidadAndTobago = Parse("+1");
        public static readonly CountryPhoneCode TristanDaCunha = Parse("+290");
        public static readonly CountryPhoneCode Tunisia = Parse("+216");
        public static readonly CountryPhoneCode Turkey = Parse("+90");
        public static readonly CountryPhoneCode Turkmenistan = Parse("+993");
        public static readonly CountryPhoneCode TurksAndCaicosIslands = Parse("+1");
        public static readonly CountryPhoneCode Tuvalu = Parse("+688");
        public static readonly CountryPhoneCode Uganda = Parse("+256");
        public static readonly CountryPhoneCode Ukraine = Parse("+380");
        public static readonly CountryPhoneCode UnitedArabEmirates = Parse("+971");
        public static readonly CountryPhoneCode UnitedKingdom = Parse("+44");
        public static readonly CountryPhoneCode UnitedStates = Parse("+1");
        public static readonly CountryPhoneCode UnitedStatesMinorOutlyingIslands = Parse("+1");
        public static readonly CountryPhoneCode Uruguay = Parse("+598");
        public static readonly CountryPhoneCode Uzbekistan = Parse("+998");
        public static readonly CountryPhoneCode Vanuatu = Parse("+678");
        public static readonly CountryPhoneCode Venezuela = Parse("+58");
        public static readonly CountryPhoneCode Vietnam = Parse("+84");
        public static readonly CountryPhoneCode VirginIslandsBritish = Parse("+1");
        public static readonly CountryPhoneCode VirginIslandsUs = Parse("+1");
        public static readonly CountryPhoneCode WallisAndFutuna = Parse("+681");
        public static readonly CountryPhoneCode Yemen = Parse("+967");
        public static readonly CountryPhoneCode Zambia = Parse("+260");
        public static readonly CountryPhoneCode Zimbabwe = Parse("+263");
    }
}
