// Copyright (c) Microsoft Corporation. All rights reserved.
// Licensed under the MIT License. See License.txt in the project root for license information.
namespace Microsoft.Azure.Management.Sql.Fluent
{
    using System.Threading;
    using System.Threading.Tasks;
    using Microsoft.Azure.Management.ResourceManager.Fluent.Core;
    using Microsoft.Azure.Management.Sql.Fluent.Models;
    using System.Collections.Generic;
    using System;

    internal partial class RecommendedElasticPoolImpl 
    {
        /// <summary>
        /// Gets the name of the resource.
        /// </summary>
        string Microsoft.Azure.Management.ResourceManager.Fluent.Core.IHasName.Name
        {
            get
            {
                return this.Name();
            }
        }

        /// <summary>
        /// Gets the edition of the Azure SQL Recommended Elastic Pool. The
        /// ElasticPoolEdition enumeration contains all the valid editions.
        /// Possible values include: 'Basic', 'Standard', 'Premium'.
        /// </summary>
        ElasticPoolEdition Microsoft.Azure.Management.Sql.Fluent.IRecommendedElasticPoolBeta.DatabaseEdition
        {
            get
            {
                return this.DatabaseEdition();
            }
        }

        /// <summary>
        /// Fetches list of databases by making call to Azure.
        /// </summary>
        /// <return>List of the databases in recommended elastic pool.</return>
        System.Collections.Generic.IReadOnlyList<Microsoft.Azure.Management.Sql.Fluent.ISqlDatabase> Microsoft.Azure.Management.Sql.Fluent.IRecommendedElasticPool.ListDatabases()
        {
            return this.ListDatabases();
        }

        /// <summary>
        /// Fetches list of metrics information by making call to Azure.
        /// </summary>
        /// <return>List of the databases in recommended elastic pool.</return>
        System.Collections.Generic.IReadOnlyList<Microsoft.Azure.Management.Sql.Fluent.IRecommendedElasticPoolMetric> Microsoft.Azure.Management.Sql.Fluent.IRecommendedElasticPool.ListMetrics()
        {
            return this.ListMetrics();
        }

        /// <summary>
        /// Gets the observation period start (ISO8601 format).
        /// </summary>
        System.DateTime? Microsoft.Azure.Management.Sql.Fluent.IRecommendedElasticPool.ObservationPeriodStart
        {
            get
            {
                return this.ObservationPeriodStart();
            }
        }

        /// <summary>
        /// Gets the minimum DTU for the database.
        /// </summary>
        double Microsoft.Azure.Management.Sql.Fluent.IRecommendedElasticPool.DatabaseDtuMin
        {
            get
            {
                return this.DatabaseDtuMin();
            }
        }

        /// <summary>
        /// Gets the DTU for the SQL Azure Recommended Elastic Pool.
        /// </summary>
        double Microsoft.Azure.Management.Sql.Fluent.IRecommendedElasticPool.Dtu
        {
            get
            {
                return this.Dtu();
            }
        }

        /// <summary>
        /// Gets the maximum DTU for the database.
        /// </summary>
        double Microsoft.Azure.Management.Sql.Fluent.IRecommendedElasticPool.DatabaseDtuMax
        {
            get
            {
                return this.DatabaseDtuMax();
            }
        }

        /// <summary>
        /// Get a specific database in the recommended database.
        /// </summary>
        /// <param name="databaseName">Name of the database to be fetched.</param>
        /// <return>Information on the database recommended in recommended elastic pool.</return>
        Microsoft.Azure.Management.Sql.Fluent.ISqlDatabase Microsoft.Azure.Management.Sql.Fluent.IRecommendedElasticPool.GetDatabase(string databaseName)
        {
            return this.GetDatabase(databaseName);
        }

        /// <summary>
        /// Gets maximum observed storage in megabytes.
        /// </summary>
        double Microsoft.Azure.Management.Sql.Fluent.IRecommendedElasticPool.MaxObservedStorageMB
        {
            get
            {
                return this.MaxObservedStorageMB();
            }
        }

        /// <summary>
        /// Gets maximum observed DTU.
        /// </summary>
        double Microsoft.Azure.Management.Sql.Fluent.IRecommendedElasticPool.MaxObservedDtu
        {
            get
            {
                return this.MaxObservedDtu();
            }
        }

        /// <summary>
        /// Get a specific database in the recommended database.
        /// </summary>
        /// <param name="databaseName">Name of the database to be fetched.</param>
        /// <return>A representation of the deferred computation to get the database in the recommended elastic pool.</return>
        async Task<Microsoft.Azure.Management.Sql.Fluent.ISqlDatabase> Microsoft.Azure.Management.Sql.Fluent.IRecommendedElasticPoolBeta.GetDatabaseAsync(string databaseName, CancellationToken cancellationToken)
        {
            return await this.GetDatabaseAsync(databaseName, cancellationToken);
        }

        /// <summary>
        /// Gets name of the SQL Server to which this database belongs.
        /// </summary>
        string Microsoft.Azure.Management.Sql.Fluent.IRecommendedElasticPool.SqlServerName
        {
            get
            {
                return this.SqlServerName();
            }
        }

        /// <summary>
        /// Gets the list of Azure SQL Databases in this pool. Expanded property.
        /// </summary>
        System.Collections.Generic.IReadOnlyList<Microsoft.Azure.Management.Sql.Fluent.ISqlDatabase> Microsoft.Azure.Management.Sql.Fluent.IRecommendedElasticPool.Databases
        {
            get
            {
                return this.Databases();
            }
        }

        /// <summary>
        /// Gets the observation period start (ISO8601 format).
        /// </summary>
        System.DateTime? Microsoft.Azure.Management.Sql.Fluent.IRecommendedElasticPool.ObservationPeriodEnd
        {
            get
            {
                return this.ObservationPeriodEnd();
            }
        }

        /// <summary>
        /// Gets storage size in megabytes.
        /// </summary>
        double Microsoft.Azure.Management.Sql.Fluent.IRecommendedElasticPool.StorageMB
        {
            get
            {
                return this.StorageMB();
            }
        }

        /// <summary>
        /// Fetches list of databases by making call to Azure.
        /// </summary>
        /// <return>A representation of the deferred computation of the databases in this recommended elastic pool.</return>
        async Task<IReadOnlyList<Microsoft.Azure.Management.Sql.Fluent.ISqlDatabase>> Microsoft.Azure.Management.Sql.Fluent.IRecommendedElasticPoolBeta.ListDatabasesAsync(CancellationToken cancellationToken)
        {
            return await this.ListDatabasesAsync(cancellationToken);
        }

        /// <summary>
        /// Gets the resource ID string.
        /// </summary>
        string Microsoft.Azure.Management.ResourceManager.Fluent.Core.IHasId.Id
        {
            get
            {
                return this.Id();
            }
        }

        /// <summary>
        /// Gets the name of the resource group.
        /// </summary>
        string Microsoft.Azure.Management.ResourceManager.Fluent.Core.IHasResourceGroup.ResourceGroupName
        {
            get
            {
                return this.ResourceGroupName();
            }
        }

        /// <summary>
        /// Gets the manager client of this resource type.
        /// </summary>
        Microsoft.Azure.Management.Sql.Fluent.ISqlManager Microsoft.Azure.Management.ResourceManager.Fluent.Core.IHasManager<Microsoft.Azure.Management.Sql.Fluent.ISqlManager>.Manager
        {
            get
            {
                return this.Manager();
            }
        }
    }
}